@@ -136,6 +136,7 @@ Writer_flush_internal(Writer *self) {
       return 0;
     }
     Py_DECREF(ret);
+    self->writebuf_start = 0;
   }
   return 1;
 }
@@ -175,6 +176,9 @@ Writer___exit__(Writer *self, PyObject *args) {
   Writer_flush_internal(self);
   if (PyObject_HasAttrString(self->fileobj, "close")) {
     PyObject *ret = PyObject_CallMethod(self->fileobj, "close", NULL);
+    if (!ret) {
+      return NULL;
+    }
     Py_DECREF(ret);
   }
   Py_RETURN_NONE;
@@ -192,7 +196,6 @@ Writer_writestr(Writer *self, PyObject *str) {
   while (i != size) {
     if (self->writebuf_start == self->writebuf_cap) {
       Writer_flush_internal(self);
-      self->writebuf_start = 0;
     }
 
     self->writebuf[(self->writebuf_start)++] = buf[i++];
@@ -207,6 +210,7 @@ Writer_writecell(Writer *self, PyObject *cell,
   unsigned char free_cellstr = 0;
   unsigned char free_replaced = 0;
   PyObject *cellstr = NULL;
+  PyObject *replaced = NULL;
 
   if (!first_cell && !Writer_writestr(self, comma_string))
     goto error_exit;
@@ -228,9 +232,7 @@ Writer_writecell(Writer *self, PyObject *cell,
   }
 
   if (need_escape && !Writer_writestr(self, quote_string)) goto error_exit;
-  PyObject *replaced = PyUnicode_Replace(cellstr,
-                                         quote_string, twoquote_string,
-                                         -1);
+  replaced = PyUnicode_Replace(cellstr, quote_string, twoquote_string, -1);
   if (!replaced) goto error_exit;
   free_replaced = 1;
   if (!Writer_writestr(self, replaced)) goto error_exit;
